/**
 * Copyright (C) 2001-2004
 * - France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors: P. Dechamboux, S. Chassande-Barrioz
 *
 */

package org.objectweb.perseus.jndi;

import java.io.FileInputStream;
import java.lang.reflect.Method;
import java.util.Enumeration;
import java.util.Properties;

import javax.naming.Context;

/**
 * This class permits to load a properties file. The values are 
 * reachable via to type: Context or Properties
 */
public class PropertiesLoader {

	/**
	 * This field is the name of the properties file.
	 */
	protected String fileName = null;

	/**
	 * This field is the reference to a context implementation
	 * which contains the values load from the properties file.
	 */
	protected Context context = null;
	
	/**
	 * This field is the class which must be used as implementation
	 * of the Context interface.
	 */
	protected Class contextImpl = null;

	/**
	 * This field is the reference to a Properties implementation
	 * which contains the values load from the properties file.
	 */
	protected Properties properties = new Properties();

	public PropertiesLoader() {
	}

	/**
	 * This contructor permits to specify which implementation of the
	 * Context interface must be used.
	 * This constructor throws an exception when the specified class
	 * isn't reachable or doesn't implement the Context interface.
	 */
	public PropertiesLoader(Class _contextImpl)
		throws ClassCastException, ClassNotFoundException,
			InstantiationException, IllegalAccessException {

		context = (Context) _contextImpl.newInstance();
		contextImpl = _contextImpl;
	}
	

	public synchronized Context getContext() { return context; }
	public synchronized void setContext(Context ctx) { context=ctx; }
	
	public synchronized String getFileName() { return fileName; }
	public synchronized void setFileName(String _fileName) { fileName = _fileName; }

	public synchronized Properties getProperties() { return properties; }
	public synchronized void setProperties(Properties p) { properties=p; }
	
	/**
	 * This method load the values of the properties file which 
	 * the name is the parameter value.
	 */
	public synchronized void load(String _fileName) throws Exception {
		fileName = _fileName;
		reload();
	}
	
	/**
	 * This method cleans all values of the current object, and reload 
	 * the new values from the current file(fileName field).
	 */
	public synchronized void reload() throws Exception {
		// Load file in properties field
		FileInputStream is = new FileInputStream(fileName);
		properties.clear();
		properties.load(is);
		
		// Full the context field if a context implementation class
		// is specified
		if (contextImpl!=null) {
			context = (Context) contextImpl.newInstance();
			Enumeration e = properties.propertyNames();
			String key = null;
			while ( e.hasMoreElements() ) {
				key = (String) e.nextElement();
				context.bind( key, properties.getProperty( key, "") );
			}
		}
	}
	
	/**
	 * This method clean the current object. No value is accessible
	 * until the next load call.
	 */
	public synchronized void clear() throws Exception {
		properties = null;
		context = null;
		fileName = null;
	}


	/**
	 *
	 */
	public synchronized void invokeSetters( Object o) throws Exception {
	
		Enumeration enum = properties.propertyNames();
		while ( enum.hasMoreElements() ){
			String fieldName = (String) enum.nextElement();
			String methodName = "set"+fieldName;
			Method[] m = o.getClass().getMethods();
			int i=0;
			while (i<m.length 
				&& (! m[i].getName().equals(methodName) 
				|| m[i].getParameterTypes().length!=1 )
				) {
				i++;
    	    }
    	    if (i<m.length) {
				Class [] paramtype = m[i].getParameterTypes();
				Object[] param = new Object[1];

				if (paramtype[0].equals(java.lang.Integer.TYPE) ||
					paramtype[0].equals(java.lang.Integer.class)){
					param[0]  = new Integer(properties.getProperty(fieldName,""));

				} else if (paramtype[0].equals(java.lang.Boolean.TYPE) ||
					paramtype[0].equals(java.lang.Boolean.class) ){
					param[0]  = new Boolean(properties.getProperty(fieldName,""));

				} else if (paramtype[0].equals(java.lang.Double.TYPE) || 
					paramtype[0].equals(java.lang.Double.class) ){
					param[0]  = new Double(properties.getProperty(fieldName,""));

				} else if (paramtype[0].equals(java.lang.Byte.TYPE) ||
					paramtype[0].equals(java.lang.Byte.class) ){
					param[0]  = new Byte(properties.getProperty(fieldName,""));
		    	
				} else if (paramtype[0].equals(java.lang.Short.TYPE) ||
					paramtype[0].equals(java.lang.Short.class)){
					param[0]  = new Short(properties.getProperty(fieldName,""));
		    	
				} else if (paramtype[0].equals(java.lang.Long.TYPE) ||
					paramtype[0].equals(java.lang.Long.class) ){
					param[0]  = new Long(properties.getProperty(fieldName,""));
		    	
				} else if (paramtype[0].equals(java.lang.Float.TYPE) ||
					paramtype[0].equals(java.lang.Float.class) ){
					param[0]  = new Float(properties.getProperty(fieldName,""));
		    	
				} else if (paramtype[0].equals(java.lang.String.class)){
					param[0]  = properties.getProperty(fieldName,"");
		    	
				} else {
					throw new Exception("incorrect type for setter method ");
				}
				// invocation of the corresponding setter method on 
				// ManagedConnectionFactory instance
				m[i].invoke(o,param);
		    } else {
		        throw new Exception("No method "+methodName+" provided by "+o);
	    	}
		}
	}
}
